/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.se.eip.patterns;

import java.util.logging.Level;

import javax.jbi.JBIException;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;

import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.util.UtilFactory;
import org.ow2.petals.se.eip.ExchangeContext;
import org.ow2.petals.se.eip.async.CommonAsyncContext;
import org.w3c.dom.Document;

/**
 * Provide a management of the source exchange response from the target
 * exchange: we must adapt the response to the MEP of the source exchange
 * 
 * @author Frederic Gardes
 */
public abstract class AbstractMEPBridgePattern extends AbstractPattern {

    protected void processResponse(final Exchange exchange, final ExchangeContext context,
            final CommonAsyncContext asyncContext) {
        NormalizedMessage receivedMessage = null;
        try {
            // process the response
            receivedMessage = PatternHelper.processSentExchange(exchange, context);
            if (!asyncContext.isTerminated()) {
                // For the source exchange, use the original MEP
                if (this.logger != null && this.logger.isLoggable(Level.FINE)) {
                    this.logger.fine(this.getName() + ": terminate the pattern");
                }
                if (receivedMessage != null) {
                    if (receivedMessage instanceof Fault) {
                        if (asyncContext.isFaultToException()) {
                            asyncContext.getOriginalExchange().setError(
                                    new JBIException(UtilFactory.getSourceUtil().createString(
                                            receivedMessage.getContent())));
                        } else if (asyncContext.getOriginalExchange().isInOnlyPattern()) {
                            asyncContext.getOriginalExchange().setDoneStatus();
                        } else {
                            asyncContext.getOriginalExchange().setFault((Fault) receivedMessage);
                        }
                    } else {
                        if (asyncContext.getOriginalExchange().isInOnlyPattern()) {
                            asyncContext.getOriginalExchange().setDoneStatus();
                        } else {
                            if (asyncContext.getOriginalExchange().isRobustInOnlyPattern()) {
                                asyncContext.getOriginalExchange().setDoneStatus();
                            } else {
                                asyncContext.getOriginalExchange().setOutMessage(receivedMessage);
                            }
                        }
                    }
                } else {
                    if (asyncContext.getOriginalExchange().getFault() == null
                            && ExchangeStatus.ACTIVE.equals(asyncContext.getOriginalExchange()
                                    .getStatus())) {
                        if (!asyncContext.getOriginalExchange().isInOutPattern()) {
                            asyncContext.getOriginalExchange().setDoneStatus();
                        } else {
                            final Document document = PatternHelper.createResultDocument(this
                                    .getNameSpace());
                            document.getDocumentElement().setTextContent(
                                    this.getName() + ": no content into the final target exchange");
                            asyncContext.getOriginalExchange().getOutMessage().setContent(
                                    UtilFactory.getSourceUtil().createStreamSource(document));
                        }
                    }
                }
                // terminate the original exchange
                context.send(asyncContext.getOriginalExchange());
                asyncContext.setTerminated();
            }
        } catch (final JBIException e) {
            if (!asyncContext.isTerminated()) {
                asyncContext.getOriginalExchange().setError(new Exception(e));
                // in this case, it's an error, terminate the original exchange
                try {
                    context.send(asyncContext.getOriginalExchange());
                } catch (final MessagingException e1) {
                    this.logger.warning(this.getName()
                            + ": unable to close the source exchange on error status");
                } finally {
                    asyncContext.setTerminated();
                }
            }
        }
    }
}
